package DZT::Sample;

1;
__END__

=head1 AUTHOR

John Doe

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 COPYRIGHT

Copyright 2001- John Doe

=cut
